package io.gitlab.jfronny.muscript.compiler.expr;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.Type;
import io.gitlab.jfronny.muscript.compiler.expr.annotations.CanThrow;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.*;
import io.gitlab.jfronny.muscript.compiler.expr.string.StringCoerce;
import io.gitlab.jfronny.muscript.error.TypeMismatchException;

@CanThrow
public sealed abstract class Expr<T> permits BoolExpr, DynamicExpr, NullLiteral, NumberExpr, StringExpr {
    protected Expr(int character) {
        this.character = character;
    }

    public abstract Type getResultType();
    public abstract T get(Dynamic<?> dataRoot);
    public abstract Expr<T> optimize();
    public final int character;

    public BoolExpr asBoolExpr() {
        if (this instanceof BoolExpr e) return e;
        throw new TypeMismatchException(character, Type.Boolean, getResultType());
    }

    public StringExpr asStringExpr() {
        if (this instanceof StringExpr e) return e;
        return new StringCoerce(character, this);
    }

    public NumberExpr asNumberExpr() {
        if (this instanceof NumberExpr e) return e;
        throw new TypeMismatchException(character, Type.Number, getResultType());
    }

    public abstract DynamicExpr asDynamicExpr();

    public boolean isNull() {
        return this instanceof NullLiteral;
    }

    public static BoolExpr literal(int character, boolean bool) {
        return new BoolLiteral(character, bool);
    }

    public static StringExpr literal(int character, String string) {
        return new StringLiteral(character, string);
    }

    public static NumberExpr literal(int character, double number) {
        return new NumberLiteral(character, number);
    }

    public static NullLiteral literalNull(int character) {
        return new NullLiteral(character);
    }
}
