package io.gitlab.jfronny.muscript.compiler.expr;

import io.gitlab.jfronny.commons.data.dynamic.DNull;
import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.Type;
import io.gitlab.jfronny.muscript.compiler.expr.annotations.CanThrow;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.DynamicLiteral;
import io.gitlab.jfronny.muscript.error.LocationalException;

@CanThrow
public final class NullLiteral extends Expr<Object> {
    public NullLiteral(int character) {
        super(character);
    }

    @Override
    public Type getResultType() {
        return Type.Dynamic;
    }

    @Override
    public Object get(Dynamic<?> dataRoot) {
        return null;
    }

    @Override
    public NullLiteral optimize() {
        return this;
    }

    @Override
    public DynamicExpr asDynamicExpr() {
        return new DynamicLiteral<>(character, new DNull());
    }

    @Override
    public NumberExpr asNumberExpr() {
        throw new LocationalException(character, "Attempted to convert null to a number");
    }

    @Override
    public StringExpr asStringExpr() {
        return Expr.literal(character, "null");
    }

    @Override
    public BoolExpr asBoolExpr() {
        throw new LocationalException(character, "Attempted to convert null to a boolean");
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof NullLiteral;
    }
}
