package io.gitlab.jfronny.muscript.compiler.expr.bool;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.BoolExpr;
import io.gitlab.jfronny.muscript.compiler.expr.Expr;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.BoolLiteral;

public class Or extends BoolExpr {
    private final BoolExpr left;
    private final BoolExpr right;

    public Or(int character, BoolExpr left, BoolExpr right) {
        super(character);
        this.left = left;
        this.right = right;
    }

    @Override
    public Boolean get(Dynamic<?> dataRoot) {
        return left.get(dataRoot) || right.get(dataRoot);
    }

    @Override
    public BoolExpr optimize() {
        BoolExpr left = this.left.optimize();
        BoolExpr right = this.right.optimize();
        if (left instanceof BoolLiteral literal) return literal.value ? Expr.literal(character, true) : right;
        if (right instanceof BoolLiteral literal) return literal.value ? Expr.literal(character, true) : left;
        return new Or(character, left, right);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Or or && left.equals(or.left) && right.equals(or.right);
    }
}
