package io.gitlab.jfronny.muscript.compiler.expr.common;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.BoolExpr;
import io.gitlab.jfronny.muscript.compiler.expr.Expr;

import java.util.*;

public class Equal extends BoolExpr {
    private final Expr<?> left;
    private final Expr<?> right;

    public Equal(int character, Expr<?> left, Expr<?> right) {
        super(character);
        this.left = left;
        this.right = right;
    }

    @Override
    public Boolean get(Dynamic<?> dataRoot) {
        return Objects.equals(unwrap(left.get(dataRoot)), unwrap(right.get(dataRoot)));
    }

    private Object unwrap(Object o) {
        if (o instanceof Dynamic<?> a) return unwrap(a.getValue());
        return o;
    }

    @Override
    public BoolExpr optimize() {
        Expr<?> left = this.left.optimize();
        Expr<?> right = this.right.optimize();
        if (left.equals(right)) return Expr.literal(character, true);
        return new Equal(character, left, right);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Equal equal && left.equals(equal.left) && right.equals(equal.right);
    }
}
