package io.gitlab.jfronny.muscript.compiler.expr.common.conditional;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.BoolExpr;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.BoolLiteral;

public class BoolConditional extends BoolExpr {
    public final BoolExpr condition;
    public final BoolExpr trueExpr;
    public final BoolExpr falseExpr;

    public BoolConditional(int character, BoolExpr condition, BoolExpr trueExpr, BoolExpr falseExpr) {
        super(character);
        this.condition = condition;
        this.trueExpr = trueExpr;
        this.falseExpr = falseExpr;
    }

    @Override
    public Boolean get(Dynamic<?> dataRoot) {
        return condition.get(dataRoot) ? trueExpr.get(dataRoot) : falseExpr.get(dataRoot);
    }

    @Override
    public BoolExpr optimize() {
        BoolExpr condition = this.condition.optimize();
        BoolExpr trueExpr = this.trueExpr.optimize();
        BoolExpr falseExpr = this.falseExpr.optimize();
        if (condition instanceof BoolLiteral literal) return literal.value ? trueExpr : falseExpr;
        if (trueExpr.equals(falseExpr)) return trueExpr;
        return new BoolConditional(character, condition, trueExpr, falseExpr);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof BoolConditional conditional
                && condition.equals(conditional.condition)
                && trueExpr.equals(conditional.trueExpr)
                && falseExpr.equals(conditional.falseExpr);
    }
}
