package io.gitlab.jfronny.muscript.compiler.expr.common.conditional;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.*;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.BoolLiteral;

public class NumberConditional extends NumberExpr {
    public final BoolExpr condition;
    public final NumberExpr trueExpr;
    public final NumberExpr falseExpr;

    public NumberConditional(int character, BoolExpr condition, NumberExpr trueExpr, NumberExpr falseExpr) {
        super(character);
        this.condition = condition;
        this.trueExpr = trueExpr;
        this.falseExpr = falseExpr;
    }

    @Override
    public Double get(Dynamic<?> dataRoot) {
        return condition.get(dataRoot) ? trueExpr.get(dataRoot) : falseExpr.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        BoolExpr condition = this.condition.optimize();
        NumberExpr trueExpr = this.trueExpr.optimize();
        NumberExpr falseExpr = this.falseExpr.optimize();
        if (condition instanceof BoolLiteral literal) return literal.value ? trueExpr : falseExpr;
        if (trueExpr.equals(falseExpr)) return trueExpr;
        return new NumberConditional(character, condition, trueExpr, falseExpr);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof NumberConditional conditional
                && condition.equals(conditional.condition)
                && trueExpr.equals(conditional.trueExpr)
                && falseExpr.equals(conditional.falseExpr);
    }
}
