package io.gitlab.jfronny.muscript.compiler.expr.dynamic;

import io.gitlab.jfronny.commons.data.dynamic.*;
import io.gitlab.jfronny.muscript.compiler.expr.*;
import io.gitlab.jfronny.muscript.compiler.expr.annotations.*;

import java.util.*;
import java.util.stream.Collectors;

@CanThrow @UncheckedDynamic
public class Call extends DynamicExpr {
    private final DynamicExpr left;
    private final List<DynamicExpr> args;

    public Call(int character, DynamicExpr left, List<DynamicExpr> args) {
        super(character);
        this.left = left;
        this.args = args;
    }

    @Override
    public Dynamic<?> get(Dynamic<?> dataRoot) {
        try {
            return left.get(dataRoot).asCallable().getValue().apply(DFinal.of(args.stream().map(e -> e.get(dataRoot)).toArray(Dynamic[]::new)));
        } catch (DynamicTypeConversionException e) {
            throw e.locational(character);
        }
    }

    @Override
    public DynamicExpr optimize() {
        DynamicExpr left = this.left.optimize();
        List<DynamicExpr> args = new ArrayList<>();
        for (DynamicExpr arg : this.args) args.add(arg.optimize());
        return new Call(character, left, args);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Call call && left.equals(call.left) && args.equals(call.args);
    }
}
