package io.gitlab.jfronny.muscript.compiler.expr.dynamic;

import io.gitlab.jfronny.commons.data.dynamic.*;
import io.gitlab.jfronny.muscript.compiler.Type;
import io.gitlab.jfronny.muscript.compiler.expr.DynamicExpr;
import io.gitlab.jfronny.muscript.compiler.expr.Expr;
import io.gitlab.jfronny.muscript.compiler.expr.annotations.*;
import io.gitlab.jfronny.muscript.error.TypeMismatchException;

@CanThrow
@UncheckedDynamic
public class Get extends DynamicExpr {
    private final DynamicExpr left;
    private final Expr<?> name;

    public Get(int character, DynamicExpr left, Expr<?> name) {
        super(character);
        this.left = left;
        this.name = name;
        if (name.getResultType() != Type.String && name.getResultType() != Type.Number && name.getResultType() != Type.Dynamic) {
            throw new TypeMismatchException(character, Type.String, name.getResultType(), "Name must be either a string or a number");
        }
    }

    @Override
    public Dynamic<?> get(Dynamic<?> dataRoot) {
        Dynamic<?> left = this.left.get(dataRoot);
        if (left instanceof DObject o) {
            return o.get(name.asStringExpr().get(dataRoot));
        } else if (left instanceof DList l) {
            return l.get(name.asNumberExpr().get(dataRoot).intValue());
        }
        throw new DynamicTypeConversionException("object or list").locational(character);
    }

    @Override
    public DynamicExpr optimize() {
        DynamicExpr left = this.left.optimize();
        Expr<?> name = this.name.optimize();
        return new Get(character, left, name);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Get get && left.equals(get.left) && name.equals(get.name);
    }
}
