package io.gitlab.jfronny.muscript.compiler.expr.dynamic.unpack;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.commons.data.dynamic.DynamicTypeConversionException;
import io.gitlab.jfronny.muscript.compiler.expr.DynamicExpr;
import io.gitlab.jfronny.muscript.compiler.expr.StringExpr;
import io.gitlab.jfronny.muscript.compiler.expr.annotations.CanThrow;
import io.gitlab.jfronny.muscript.compiler.expr.annotations.UncheckedDynamic;
import io.gitlab.jfronny.muscript.compiler.expr.dynamic.DynamicCoerce;

@CanThrow
@UncheckedDynamic
public class StringUnpack extends StringExpr {
    public final DynamicExpr inner;

    public StringUnpack(int character, DynamicExpr inner) {
        super(character);
        this.inner = inner;
    }

    @Override
    public String get(Dynamic<?> dataRoot) {
        try {
            return inner.get(dataRoot).asString().getValue();
        } catch (DynamicTypeConversionException e) {
            throw e.locational(character);
        }
    }

    @Override
    public StringExpr optimize() {
        DynamicExpr inner = this.inner.optimize();
        if (inner instanceof DynamicCoerce coerce) return coerce.inner.asStringExpr();
        return new StringUnpack(character, inner);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof StringUnpack unpack && inner.equals(unpack.inner);
    }
}
