package io.gitlab.jfronny.muscript.compiler.expr.number.compare;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.*;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.NumberLiteral;
import io.gitlab.jfronny.muscript.compiler.expr.number.math.*;

public class Less extends BoolExpr {
    private final NumberExpr left;
    private final NumberExpr right;

    public Less(int character, NumberExpr left, NumberExpr right) {
        super(character);
        this.left = left;
        this.right = right;
    }

    @Override
    public Boolean get(Dynamic<?> dataRoot) {
        return left.get(dataRoot) < right.get(dataRoot);
    }

    @Override
    public BoolExpr optimize() {
        NumberExpr left = this.left.optimize();
        NumberExpr right = this.right.optimize();
        if (left instanceof NumberLiteral litL && right instanceof NumberLiteral litR) return Expr.literal(character, litL.value < litR.value);
        if (left instanceof Divide divide) return new Less(character, divide.dividend, new Multiply(divide.character, divide.divisor, right)).optimize();
        if (left instanceof Invert invert) return new Greater(character, invert.inner, new Invert(right.character, right)).optimize();
        if (left instanceof Minus minus) return new Less(character, minus.minuend, new Plus(minus.character, minus.subtrahend, right)).optimize();
        // Modulo is left out because it is too complicated for this naive impl
        // Multiply is left out since it would transform into a division and may be 0
        if (left instanceof Plus plus) return new Less(character, plus.augend, new Minus(plus.character, plus.addend, right)).optimize();
        // Power is left out because it can't be transformed cleanly either
        return new Less(character, left, right);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Less less && left.equals(less.left) && right.equals(less.right);
    }
}
