package io.gitlab.jfronny.muscript.compiler.expr.number.math;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.Expr;
import io.gitlab.jfronny.muscript.compiler.expr.NumberExpr;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.NumberLiteral;

public class Divide extends NumberExpr {
    public final NumberExpr dividend;
    public final NumberExpr divisor;

    public Divide(int character, NumberExpr dividend, NumberExpr divisor) {
        super(character);
        this.dividend = dividend;
        this.divisor = divisor;
    }

    @Override
    public Double get(Dynamic<?> dataRoot) {
        return dividend.get(dataRoot) / divisor.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        NumberExpr dividend = this.dividend.optimize();
        NumberExpr divisor = this.divisor.optimize();
        if (dividend instanceof NumberLiteral litL && divisor instanceof NumberLiteral litR) return Expr.literal(character, litL.value / litR.value);
        if (dividend instanceof Divide divide && divide.dividend instanceof NumberLiteral literal) return new Divide(character, divide.dividend, new Multiply(divide.character, divisor, literal)).optimize();
        return new Divide(character, dividend, divisor);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Divide divide && dividend.equals(divide.dividend) && divisor.equals(divide.divisor);
    }
}
