package io.gitlab.jfronny.muscript.compiler.expr.number.math;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.Expr;
import io.gitlab.jfronny.muscript.compiler.expr.NumberExpr;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.NumberLiteral;

public class Minus extends NumberExpr {
    public final NumberExpr minuend;
    public final NumberExpr subtrahend;

    public Minus(int character, NumberExpr minuend, NumberExpr subtrahend) {
        super(character);
        this.minuend = minuend;
        this.subtrahend = subtrahend;
    }

    @Override
    public Double get(Dynamic<?> dataRoot) {
        return minuend.get(dataRoot) - subtrahend.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        NumberExpr minuend = this.minuend.optimize();
        NumberExpr subtrahend = this.subtrahend.optimize();
        if (minuend instanceof NumberLiteral litM && subtrahend instanceof NumberLiteral litS) return Expr.literal(character, litM.value - litS.value);
        if (minuend instanceof Minus minus) return new Minus(character, minus.minuend, new Plus(minus.character, minus.subtrahend, subtrahend)).optimize();
        return new Minus(character, minuend, subtrahend);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Minus minus && minuend.equals(minus.minuend) && subtrahend.equals(minus.subtrahend);
    }
}
