package io.gitlab.jfronny.muscript.compiler.expr.number.math;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.Expr;
import io.gitlab.jfronny.muscript.compiler.expr.NumberExpr;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.NumberLiteral;

public class Modulo extends NumberExpr {
    private final NumberExpr dividend;
    private final NumberExpr divisor;

    public Modulo(int character, NumberExpr dividend, NumberExpr divisor) {
        super(character);
        this.dividend = dividend;
        this.divisor = divisor;
    }

    @Override
    public Double get(Dynamic<?> dataRoot) {
        return dividend.get(dataRoot) % divisor.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        NumberExpr dividend = this.dividend.optimize();
        NumberExpr divisor = this.divisor.optimize();
        if (dividend instanceof NumberLiteral litD && divisor instanceof NumberLiteral litS) return Expr.literal(character, litD.value % litS.value);
        return new Modulo(character, dividend, divisor);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Modulo modulo && dividend.equals(modulo.dividend) && divisor.equals(modulo.divisor);
    }
}
