package io.gitlab.jfronny.muscript.compiler.expr.number.math;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.Expr;
import io.gitlab.jfronny.muscript.compiler.expr.NumberExpr;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.NumberLiteral;

public class Multiply extends NumberExpr {
    public final NumberExpr multiplier;
    public final NumberExpr multiplicand;

    public Multiply(int character, NumberExpr multiplier, NumberExpr multiplicand) {
        super(character);
        this.multiplier = multiplier;
        this.multiplicand = multiplicand;
    }

    @Override
    public Double get(Dynamic<?> dataRoot) {
        return multiplier.get(dataRoot) * multiplicand.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        NumberExpr multiplier = this.multiplier.optimize();
        NumberExpr multiplicand = this.multiplicand.optimize();
        if (multiplier instanceof NumberLiteral litEr && multiplicand instanceof NumberLiteral litAnd) return Expr.literal(character, litEr.value * litAnd.value);
        return new Modulo(character, multiplier, multiplicand);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Multiply multiply && multiplier.equals(multiply.multiplier) && multiplicand.equals(multiply.multiplicand);
    }
}
