package io.gitlab.jfronny.muscript.compiler.expr.number.math;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.Expr;
import io.gitlab.jfronny.muscript.compiler.expr.NumberExpr;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.NumberLiteral;

public class Power extends NumberExpr {
    private final NumberExpr base;
    private final NumberExpr exponent;

    public Power(int character, NumberExpr base, NumberExpr exponent) {
        super(character);
        this.base = base;
        this.exponent = exponent;
    }

    @Override
    public Double get(Dynamic<?> dataRoot) {
        return Math.pow(base.get(dataRoot), exponent.get(dataRoot));
    }

    @Override
    public NumberExpr optimize() {
        NumberExpr base = this.base.optimize();
        NumberExpr exponent = this.exponent.optimize();
        if (base instanceof NumberLiteral litB && exponent instanceof NumberLiteral litE) return Expr.literal(character, Math.pow(litB.value, litE.value));
        if (exponent instanceof NumberLiteral exp && base instanceof Multiply multiply) {
            if (multiply.multiplier instanceof NumberLiteral literal)
                return new Multiply(multiply.character,
                    Expr.literal(character, Math.pow(literal.value, exp.value)),
                    new Power(character, multiply.multiplicand, exp));
            if (multiply.multiplicand instanceof NumberLiteral literal)
                return new Multiply(multiply.character,
                    Expr.literal(character, Math.pow(literal.value, exp.value)),
                    new Power(character, multiply.multiplier, exp));
        }
        return new Power(character, base, exponent);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Power power && base.equals(power.base) && exponent.equals(power.exponent);
    }
}
