package io.gitlab.jfronny.muscript.compiler.expr.string;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.*;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.StringLiteral;

public class Concatenate extends StringExpr {
    private final StringExpr left;
    private final StringExpr right;

    public Concatenate(int character, StringExpr left, StringExpr right) {
        super(character);
        this.left = left;
        this.right = right;
    }

    @Override
    public String get(Dynamic<?> dataRoot) {
        return left.get(dataRoot) + right.get(dataRoot);
    }

    @Override
    public StringExpr optimize() {
        StringExpr left = this.left.optimize();
        StringExpr right = this.right.optimize();
        if (left instanceof StringLiteral litL && right instanceof StringLiteral litR) return Expr.literal(character, litL.value + litR.value);
        if (right instanceof StringLiteral litR && left instanceof Concatenate concatenate && concatenate.right instanceof StringLiteral litL) {
            return new Concatenate(character, concatenate.left, Expr.literal(concatenate.character, litL.value + litR.value));
        }
        if (left instanceof StringLiteral litL && right instanceof Concatenate concatenate && concatenate.left instanceof StringLiteral litR) {
            return new Concatenate(character, Expr.literal(concatenate.character, litL.value + litR.value), concatenate.right);
        }
        return new Concatenate(character, left, right);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Concatenate concatenate && left.equals(concatenate.left) && right.equals(concatenate.right);
    }
}
