package io.gitlab.jfronny.muscript.compiler.expr.string;

import io.gitlab.jfronny.commons.StringFormatter;
import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.expr.*;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.*;

public class StringCoerce extends StringExpr {
    private final Expr<?> inner;

    public StringCoerce(int character, Expr<?> inner) {
        super(character);
        this.inner = inner;
    }

    @Override
    public String get(Dynamic<?> dataRoot) {
        return StringFormatter.toString(inner.get(dataRoot));
    }

    @Override
    public StringExpr optimize() {
        Expr<?> inner = this.inner.optimize();
        if (inner instanceof NullLiteral) return Expr.literal(character, "null");
        if (inner instanceof BoolLiteral expr) return Expr.literal(character, StringFormatter.toString(expr.value));
        if (inner instanceof DynamicLiteral<?> expr) return Expr.literal(character, StringFormatter.toString(expr.value));
        if (inner instanceof NumberLiteral expr) return Expr.literal(character, StringFormatter.toString(expr.value));
        if (inner instanceof StringExpr expr) return expr;
        return new StringCoerce(character, inner);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof StringCoerce coerce && inner.equals(coerce.inner);
    }
}
