package io.gitlab.jfronny.muscript.compiler.expr.unresolved;

import io.gitlab.jfronny.commons.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.compiler.Type;
import io.gitlab.jfronny.muscript.compiler.expr.*;
import io.gitlab.jfronny.muscript.compiler.expr.annotations.CanThrow;
import io.gitlab.jfronny.muscript.compiler.expr.common.conditional.*;
import io.gitlab.jfronny.muscript.compiler.expr.common.literal.BoolLiteral;

@CanThrow
public class UnresolvedConditional extends DynamicExpr {
    private final BoolExpr condition;
    private final Expr<?> trueExpr;
    private final Expr<?> falseExpr;

    public UnresolvedConditional(int character, BoolExpr condition, Expr<?> trueExpr, Expr<?> falseExpr) {
        super(character);
        this.condition = condition;
        this.trueExpr = trueExpr;
        this.falseExpr = falseExpr;
    }

    @Override
    public Type getResultType() {
        return trueExpr.getResultType();
    }

    @Override
    public DynamicExpr optimize() {
        BoolExpr condition = this.condition.optimize();
        Expr<?> trueExpr = this.trueExpr.optimize();
        Expr<?> falseExpr = this.falseExpr.optimize();
        if (condition instanceof BoolLiteral literal) return literal.value ? trueExpr.asDynamicExpr() : falseExpr.asDynamicExpr();
        if (trueExpr.equals(falseExpr)) return trueExpr.asDynamicExpr();
        return new UnresolvedConditional(character, condition, trueExpr, falseExpr);
    }

    @Override
    public Dynamic<?> get(Dynamic<?> dataRoot) {
        throw new UnsupportedOperationException("Conditional was kept unresolved. This is not supported!");
    }

    @Override
    public DynamicExpr asDynamicExpr() {
        return new DynamicConditional(character, condition, trueExpr.asDynamicExpr(), falseExpr.asDynamicExpr());
    }

    @Override
    public BoolExpr asBoolExpr() {
        return new BoolConditional(character, condition, trueExpr.asBoolExpr(), falseExpr.asBoolExpr());
    }

    @Override
    public StringExpr asStringExpr() {
        return new StringConditional(character, condition, trueExpr.asStringExpr(), falseExpr.asStringExpr());
    }

    @Override
    public NumberExpr asNumberExpr() {
        return new NumberConditional(character, condition, trueExpr.asNumberExpr(), falseExpr.asNumberExpr());
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof UnresolvedConditional conditional
                && condition.equals(conditional.condition)
                && trueExpr.equals(conditional.trueExpr)
                && falseExpr.equals(conditional.falseExpr);
    }
}
