package io.gitlab.jfronny.commons.log;

import io.gitlab.jfronny.commons.cache.FixedSizeSet;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;
import java.util.function.Consumer;

public class MemoryLogger implements Logger, Iterable<String> {
    private final String name;
    private final Set<String> lines;

    public MemoryLogger(String name) {
        this.name = name;
        this.lines = new LinkedHashSet<>();
    }

    public MemoryLogger(String name, int size) {
        this.name = name;
        this.lines = new FixedSizeSet<>(size);
    }

    @Override
    public @Nullable String getName() {
        return name;
    }

    @Override
    public void trace(String msg) {
        lines.add("[T] " + msg);
    }

    @Override
    public void debug(String msg) {
        lines.add("[D] " + msg);
    }

    @Override
    public void info(String msg) {
        lines.add("[I] " + msg);
    }

    @Override
    public void warn(String msg) {
        lines.add("[W] " + msg);
    }

    @Override
    public void error(String msg) {
        lines.add("[E] " + msg);
    }

    @NotNull
    @Override
    public Iterator<String> iterator() {
        return lines.iterator();
    }

    @Override
    public void forEach(Consumer<? super String> action) {
        lines.forEach(action);
    }

    @Override
    public Spliterator<String> spliterator() {
        return lines.spliterator();
    }
}
