package io.gitlab.jfronny.commons.reflect;

import io.gitlab.jfronny.commons.throwable.ThrowingBiFunction;
import io.gitlab.jfronny.commons.throwable.ThrowingFunction;
import io.gitlab.jfronny.commons.throwable.ThrowingSupplier;

import java.lang.invoke.MethodHandles;
import java.util.function.*;

@SuppressWarnings("unchecked")
public class Reflect {
    @Deprecated
    public static <TOut> ThrowingSupplier<TOut, ReflectiveOperationException> getConstructor(Class<TOut> toConstruct) throws Throwable {
        Supplier<TOut> constructor = constructor(toConstruct);
        return constructor::get;
    }

    @Deprecated
    public static <TOut> ThrowingSupplier<TOut, ReflectiveOperationException> getConstructor(String targetClassName) throws Throwable {
        return Reflect.getConstructor((Class<TOut>) Class.forName(targetClassName));
    }

    @Deprecated
    public static <TIn, TOut> ThrowingFunction<TIn, TOut, ReflectiveOperationException> getConstructor(Class<TOut> toConstruct, Class<TIn> parameterType) throws Throwable {
        Function<TIn, TOut> constructor = constructor(toConstruct, parameterType);
        return constructor::apply;
    }

    @Deprecated
    public static <TIn, TOut> ThrowingFunction<TIn, TOut, ReflectiveOperationException> getConstructor(String targetClassName, Class<TIn> parameterType) throws Throwable {
        return getConstructor((Class<TOut>) Class.forName(targetClassName), parameterType);
    }

    @Deprecated
    public static <TIn1, TIn2, TOut> ThrowingBiFunction<TIn1, TIn2, TOut, ReflectiveOperationException> getConstructor(Class<TOut> toConstruct, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        BiFunction<TIn1, TIn2, TOut> constructor = constructor(toConstruct, parameterType1, parameterType2);
        return constructor::apply;
    }

    @Deprecated
    public static <TIn1, TIn2, TOut> ThrowingBiFunction<TIn1, TIn2, TOut, ReflectiveOperationException> getConstructor(String targetClassName, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        return getConstructor((Class<TOut>) Class.forName(targetClassName), parameterType1, parameterType2);
    }

    // Constructor without parameters
    public static <TOut> Supplier<TOut> constructor(Class<TOut> toConstruct) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(toConstruct);
        return LambdaFactory.supplier(lookup, lookup.unreflectConstructor(toConstruct.getDeclaredConstructor()), toConstruct);
    }

    public static <TOut> Supplier<TOut> constructor(String targetClassName) throws Throwable {
        return (Supplier<TOut>) constructor(Class.forName(targetClassName));
    }

    // Constructor with one parameter
    public static <TIn, TOut> Function<TIn, TOut> constructor(Class<TOut> toConstruct, Class<TIn> parameterType) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(toConstruct);
        return LambdaFactory.function(lookup, lookup.unreflectConstructor(toConstruct.getDeclaredConstructor(parameterType)), toConstruct, parameterType);
    }

    public static <TIn, TOut> Function<TIn, TOut> constructor(String targetClassName, Class<TIn> parameterType) throws Throwable {
        return (Function<TIn, TOut>) constructor(Class.forName(targetClassName), parameterType);
    }

    // Constructor with two parameters
    public static <TIn1, TIn2, TOut> BiFunction<TIn1, TIn2, TOut> constructor(Class<TOut> toConstruct, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(toConstruct);
        return LambdaFactory.function(lookup, lookup.unreflectConstructor(toConstruct.getDeclaredConstructor(parameterType1, parameterType2)), toConstruct, parameterType1, parameterType2);
    }

    public static <TIn1, TIn2, TOut> BiFunction<TIn1, TIn2, TOut> constructor(String targetClassName, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        return constructor((Class<TOut>) Class.forName(targetClassName), parameterType1, parameterType2);
    }

    // Procedure without parameters
    public static Runnable staticProcedure(Class<?> targetClass, String name) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.runnable(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name)));
    }

    public static Runnable staticProcedure(String targetClassName, String name) throws Throwable {
        return staticProcedure(Class.forName(targetClassName), name);
    }

    // Procedure with one parameter
    public static <TIn> Consumer<TIn> staticProcedure(Class<?> targetClass, String name, Class<TIn> parameterType) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.consumer(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name, parameterType)), parameterType);
    }

    public static <TIn> Consumer<TIn> staticProcedure(String targetClassName, String name, Class<TIn> parameterType) throws Throwable {
        return staticProcedure(Class.forName(targetClassName), name, parameterType);
    }

    // Procedure with two parameters
    public static <TIn1, TIn2> BiConsumer<TIn1, TIn2> staticProcedure(Class<?> targetClass, String name, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.consumer(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name, parameterType1, parameterType2)), parameterType1, parameterType2);
    }

    public static <TIn1, TIn2> BiConsumer<TIn1, TIn2> staticProcedure(String targetClassName, String name, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        return staticProcedure(Class.forName(targetClassName), name, parameterType1, parameterType2);
    }

    // Function without parameters
    public static <TOut> Supplier<TOut> staticFunction(Class<?> targetClass, String name, Class<TOut> returnType) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.supplier(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name)), returnType);
    }

    public static <TOut> Supplier<TOut> staticFunction(String targetClassName, String name, Class<TOut> returnType) throws Throwable {
        return staticFunction(Class.forName(targetClassName), name, returnType);
    }

    // Function with one parameter
    public static <TIn, TOut> Function<TIn, TOut> staticFunction(Class<?> targetClass, String name, Class<TOut> returnType, Class<TIn> parameterType) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.function(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name, parameterType)), returnType, parameterType);
    }

    public static <TIn, TOut> Function<TIn, TOut> staticFunction(String targetClassName, String name, Class<TOut> returnType, Class<TIn> parameterType) throws Throwable {
        return staticFunction(Class.forName(targetClassName), name, returnType, parameterType);
    }

    // Function with two parameters
    public static <TIn1, TIn2, TOut> BiFunction<TIn1, TIn2, TOut> staticFunction(Class<?> targetClass, String name, Class<TOut> returnType, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.function(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name, parameterType1, parameterType2)), returnType, parameterType1, parameterType2);
    }

    public static <TIn1, TIn2, TOut> BiFunction<TIn1, TIn2, TOut> staticFunction(String targetClassName, String name, Class<TOut> returnType, Class<TIn1> parameterType1, Class<TIn2> parameterType2) throws Throwable {
        return staticFunction(Class.forName(targetClassName), name, returnType, parameterType1, parameterType2);
    }

    // Procedure without parameters
    public static <TTarget> Consumer<TTarget> instanceProcedure(Class<TTarget> targetClass, String name) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.consumer(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name)), targetClass);
    }

    public static <TTarget> Consumer<TTarget> instanceProcedure(String targetClassName, String name) throws Throwable {
        return (Consumer<TTarget>) instanceProcedure(Class.forName(targetClassName), name);
    }

    // Procedure with one parameter
    public static <TTarget, TIn> BiConsumer<TTarget, TIn> instanceProcedure(Class<TTarget> targetClass, String name, Class<TIn> parameterType) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.consumer(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name, parameterType)), targetClass, parameterType);
    }

    public static <TTarget, TIn> BiConsumer<TTarget, TIn> instanceProcedure(String targetClassName, String name, Class<TIn> parameterType) throws Throwable {
        return (BiConsumer<TTarget, TIn>) instanceProcedure(Class.forName(targetClassName), name, parameterType);
    }

    // Function without parameters
    public static <TTarget, TOut> Function<TTarget, TOut> instanceFunction(Class<TTarget> targetClass, String name, Class<TOut> returnType) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.function(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name)), returnType, targetClass);
    }

    public static <TTarget, TOut> Function<TTarget, TOut> instanceFunction(String targetClassName, String name, Class<TOut> returnType) throws Throwable {
        return (Function<TTarget, TOut>) instanceFunction(Class.forName(targetClassName), name, returnType);
    }

    // Function with one parameter
    public static <TTarget, TIn, TOut> BiFunction<TTarget, TIn, TOut> instanceFunction(Class<TTarget> targetClass, String name, Class<TOut> returnType, Class<TIn> parameterType) throws Throwable {
        MethodHandles.Lookup lookup = LambdaFactory.lookup(targetClass);
        return LambdaFactory.function(lookup, lookup.unreflect(targetClass.getDeclaredMethod(name, parameterType)), returnType, targetClass, parameterType);
    }

    public static <TTarget, TIn, TOut> BiFunction<TTarget, TIn, TOut> instanceFunction(String targetClassName, String name, Class<TOut> returnType, Class<TIn> parameterType) throws Throwable {
        return (BiFunction<TTarget, TIn, TOut>) instanceFunction(Class.forName(targetClassName), name, returnType, parameterType);
    }
}
