package io.gitlab.jfronny.commons.tuple;

import io.gitlab.jfronny.commons.throwable.ThrowingPredicate;
import io.gitlab.jfronny.commons.throwable.ThrowingSupplier;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;
import java.util.function.Function;

public record Single<T1>(@Nullable T1 val) {
    @Contract(value = "_ -> new", pure = true)
    public static <T1> @NotNull Single<T1> of(@Nullable T1 val) {
        return new Single<>(val);
    }

    @Contract(pure = true)
    public boolean isNull() {
        return val == null;
    }

    @Contract(pure = true)
    public boolean isPresent() {
        return val != null;
    }

    @Contract(value = "-> new", pure = true)
    public @NotNull ThrowingPredicate<T1, RuntimeException> asEqualsPredicate() {
        return val == null ? Objects::isNull : val::equals;
    }

    @Contract(value = "-> new", pure = true)
    public @NotNull ThrowingPredicate<T1, RuntimeException> asInstanceEqualsPredicate() {
        return v -> v == val;
    }

    @Contract(value = "-> new", pure = true)
    public @NotNull ThrowingSupplier<T1, RuntimeException> asSupplier() {
        return () -> val;
    }

    @Contract(value = "_, -> new", pure = true)
    public <T> @NotNull Single<T> map(@NotNull Function<T1, T> mapper) {
        return new Single<>(Objects.requireNonNull(mapper).apply(val));
    }

    @Contract(pure = true)
    public @Nullable T1 get() {
        return val;
    }
}
